<?php
/**
 * Utility class
 *
 * @since 1.0
 */
namespace dodebug;

defined( 'WPINC' ) || exit;

class Util extends Instance
{
	protected static $_instance;

	/**
	 * Init Utility
	 *
	 * @since 1.0
	 * @access public
	 */
	public function init()
	{
	}

	/**
	 * Check if an array has a string or not
	 * Support $ exact match
	 *
	 * @since 1.0
	 */
	public static function str_hit_array( $needle, $haystack, $has_ttl = false )
	{
		if ( ! is_array( $haystack ) ) {
			defined( 'debug' ) && debug( "[Util] ❌ bad param in str_hit_array()!" );
			return false;
		}

		$hit = false;
		$this_ttl = 0;
		foreach( $haystack as $item ) {
			if ( ! $item ) {
				continue;
			}

			if ( $has_ttl ) {
				$this_ttl = 0;
				$item = explode( ' ', $item );
				if ( ! empty( $item[ 1 ] ) ) {
					$this_ttl = $item[ 1 ];
				}
				$item = $item[ 0 ];
			}

			if ( substr( $item, -1 ) === '$' ) {
				// do exact match
				if ( substr( $item, 0, -1 ) === $needle ) {
					$hit = $item;
					break;
				}
			}
			elseif ( substr( $item, 0, 1 ) === '^' ) {
				// match beginning
				if ( substr( $item, 1 ) === substr( $needle, 0, strlen( $item ) - 1 ) ) {
					$hit = $item;
					break;
				}
			}
			else {
				if ( strpos( $needle, $item ) !== false ) {
					$hit = $item;
					break;
				}
			}
		}

		if ( $hit ) {
			if ( $has_ttl ) {
				return array( $hit, $this_ttl );
			}

			return $hit;
		}

		return false;
	}

	/**
	 * Builds an url with an action and a nonce.
	 *
	 * @since  1.0
	 * @access public
	 */
	public static function build_url( $action, $type = false, $is_ajax = false, $page = null, $append_arr = null )
	{
		$prefix = '?';

		if ( ! $is_ajax ) {
			if ( $page ) {
				// If use admin url
				if ( $page === true ) {
					$page = 'admin.php';
				}
				else {
					if ( strpos( $page, '?' ) !== false ) {
						$prefix = '&';
					}
				}
				$combined = $page . $prefix . Router::ACTION . '=' . $action;
			}
			else {
				// Current page rebuild URL
				$params = array_map( 'sanitize_text_field', $_GET );

				if ( ! empty( $params ) ) {
					if ( isset( $params[ 'DODEBUG_ACTION' ] ) ) {
						unset( $params[ 'DODEBUG_ACTION' ] );
					}
					if ( isset( $params[ '_wpnonce' ] ) ) {
						unset( $params[ '_wpnonce' ] );
					}
					if ( ! empty( $params ) ) {
						$prefix .= http_build_query( $params ) . '&';
					}
				}
				global $pagenow;
				$combined = $pagenow . $prefix . Router::ACTION . '=' . $action;
			}
		}
		else {
			$combined = 'admin-ajax.php?action=dodebug_ajax&' . Router::ACTION . '=' . $action;
		}

		if ( is_network_admin() ) {
			$prenonce = network_admin_url( $combined );
		}
		else {
			$prenonce = admin_url( $combined );
		}
		$url = wp_nonce_url( $prenonce, $action, Router::NONCE );

		if ( $type ) {
			// Remove potential param `type` from url
			$url = parse_url( htmlspecialchars_decode( $url ) );
			parse_str( $url[ 'query' ], $query );

			$built_arr = array_merge( $query, array( Router::TYPE => $type ) );
			if ( $append_arr ) {
				$built_arr = array_merge( $built_arr, $append_arr );
			}
			$url[ 'query' ] = http_build_query( $built_arr );
			self::compatibility();
			$url = http_build_url( $url );
			$url = htmlspecialchars( $url, ENT_QUOTES, 'UTF-8' );
		}

		return $url;
	}

	/**
	 * Improve compatibility to PHP old versions
	 *
	 * @since  1.0
	 *
	 */
	public static function compatibility()
	{
		require_once DODEBUG_DIR . 'lib/php-compatibility.func.php';
	}

	/**
	 * Set seconds/timestamp to readable format
	 *
	 * @since  1.0
	 * @access public
	 */
	public static function readable_time( $seconds_or_timestamp, $timeout = 3600, $backward = true )
	{
		if ( strlen( $seconds_or_timestamp ) == 10 ) {
			$seconds = time() - $seconds_or_timestamp;
			if ( $seconds > $timeout ) {
				return date( 'm/d/Y H:i:s', $seconds_or_timestamp + get_option( 'gmt_offset' ) * 60 * 60 );
			}
		}
		else {
			$seconds = $seconds_or_timestamp;
		}
		$res = '';
		if ( $seconds > 86400 ) {
			$num = floor( $seconds / 86400 );
			$res .= $num . 'd';
			$seconds %= 86400;
		}
		if ( $seconds > 3600 ) {
			if ( $res ) {
				$res .= ', ';
			}
			$num = floor( $seconds / 3600 );
			$res .= $num . 'h';
			$seconds %= 3600;
		}
		if ( $seconds > 60 ) {
			if ( $res ) {
				$res .= ', ';
			}
			$num = floor( $seconds / 60 );
			$res .= $num . 'm';
			$seconds %= 60;
		}
		if ( $seconds > 0 ) {
			if ( $res ) {
				$res .= ' ';
			}
			$res .= $seconds . 's';
		}
		if ( ! $res ) {
			return $backward ? __( 'just now', 'dodebug' ) : __( 'right now', 'dodebug' );
		}
		$res = $backward ? sprintf( __( ' %s ago', 'dodebug' ), $res ) : $res;
		return $res;
	}

	/**
	 * Deactivate
	 *
	 * @since  1.0
	 * @access public
	 */
	public static function deactivate()
	{
		delete_transient( 'dodebug_activation_redirect' );

		Log::get_instance()->clear_log( true );
	}

	/**
	 * Uninstall clearance
	 *
	 * @since  1.0
	 * @access public
	 */
	public static function uninstall()
	{
		self::version_check( 'uninstall' );
	}

	/**
	 * Activation redirect
	 *
	 * @since  1.0
	 * @access public
	 */
	public static function activate()
	{
		if ( ! defined( 'SILENCE_INSTALL' ) ) {
			set_transient( 'dodebug_activation_redirect', true, 30 );
		}
	}

}